//==============================================================================
//
// Project : FREE DELTA - Software system for processing taxonomic description
//           coded in DELTA (DEscription Language for TAxonomy) format
//
// Module  : Identification - Function for testing item identification
//                            with tDelta class.
//
// Release : 0.10 alpha - January 2001
// Author  : Denis ZIEGLER  denis.ziegler@free.fr
//
// File    : identification.cpp
//
// Portability : C++ ANSI (DOS, Windows, Unix,...)
//
// (C) Copyright 2001 - Denis ZIEGLER
//==============================================================================

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation                                          *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>

#include "identification.h"


void message(void)
{
  cout << endl << "IDENTIFICATION" << endl;
  cout << "This function demonstrates the possibility using tDelta class for taxa" << endl;
  cout << "identification." << endl;
  cout << "In this example, the identification process has been simplified a lot." << endl;
  cout << "It is based on successive comparisons between observed characters values," << endl;
  cout << "and the items descriptions stored in the Delta files." << endl;
  cout << "Advanced features, such as values combinations, implicit values, dependant" << endl;
  cout << "characters are not implemented. There is no characters selection during" << endl;
  cout << "identification (like the 'best' command in Intkey)." << endl;
  cout << "Of course, the user interface is very simple, and should be improved in" << endl;
  cout << "a public release." << endl << endl;
  cout << "Usage :" << endl;
  cout << "To make an identification, you should have a printed version of characters" << endl;
  cout << "description (Delta chars file)." << endl;
  cout << "Select a character and a value according to the taxon you want to identify." << endl;
  cout << "After each selection, the program displays all characters values already" << endl;
  cout << "selected, and the list of remaining taxa." << endl;
  cout << "To cancel a previous selected character, select it again, and input the" << endl;
  cout << "UNKNOWN value (-1). You can also modify a previously selected value." << endl << endl;
}


void identification(tDelta *delta)
{
  double *tabval;  // values table : stores selected value for each character
  int *tabitm;     // item table : indicates for each item if it is matching
                   //              with the values stored in tabval
  int nbval,       // character number (dimension of tabval)
      nbitm,       // item number (dimension of tabitm)
      charnum,     // current character number selected
      i, j;
  double charval;  // current character value selected

  //----- Initialisation -----
  nbval = delta->chars->get_chars_nb();
  nbitm = delta->items->get_items_nb();
  tabval = new double[nbval];
  for (i=0; i<nbval; i++)
    tabval[i] = UNKNOWN;
  tabitm = new int[nbitm];
  message();
  //----- User input and search matching items -----
  while (1) {
    //--- Input character number and value
    cout << "Select a character describing the item to identify :" << endl;
    cout << "  Enter character number (0=exit) : ";
    scanf("%d", &charnum);
    if (!charnum)
      break;
    if ((charnum < 1) || (charnum > nbval)) {
      cerr << "  Invalid character number" << endl;
      continue;
    }
    cout << "  Enter character state/value (-1=unknown) : ";
    scanf("%lf", &charval);
    //--- Stores value
    tabval[charnum-1] = (charval == -1.) ? UNKNOWN : charval;
    //--- Update items table considering selected values
    for (i=0; i<nbitm; i++)
      for (j=0; j<nbval; j++) {
        if (tabval[j] == UNKNOWN)  // if no selected value
          tabitm[i] = 1;           //    item is remaining
        else                       // else item is remaining if value is matching
          tabitm[i] = delta->items->matches(i+1, j+1, &tabval[j]);
        if (!tabitm[i])  // if item is not remaining, we don't test other characters
          break;
      }
    //--- Displays selected characters and values, and remaining items list
    cout << endl << "CHARACTERS SELECTION" << endl;
    for (j=0; j<nbval; j++)
      if (tabval[j] != UNKNOWN) {
        cout << (j+1) << ". " << delta->chars->get_char_feature(j+1) << endl;
        cout << "    state/value : " << tabval[j];
        if (delta->chars->get_char_type(j+1) & CT_UM)
          cout << " - " << delta->chars->get_state(j+1, (int) tabval[j]);
        else
          if (delta->chars->get_char_type(j+1) & CT_IN)
            cout << " " << delta->chars->get_char_unit(j+1);
        cout << endl;
      }
    cout << "ITEMS REMAINING" << endl;
    for (i=0; i<nbitm; i++)
      if (tabitm[i])
        cout << delta->items->get_item_name(i+1, 0) << endl;
    cout << endl;
  }
  delete tabval;
  delete tabitm;
}
