//==============================================================================
//
// Project : FREE DELTA - Software system for processing taxonomic description
//           coded in DELTA (DEscription Language for TAxonomy) format
//
// Module  : Test program for tDelta
//
// Release : 0.20 beta - August 2001
// Author  : Denis ZIEGLER  denis.ziegler@free.fr
//
// File    : test_tdelta.cpp
//
// Portability : C++ ANSI (DOS, Windows, Unix,...)
//
// (C) Copyright 2001 - Denis ZIEGLER
//==============================================================================

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation                                          *
 ***************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <ctype.h>

#include "identification.h"


int main(int argc, char **argv)
{
  tDelta *Example;
  int i, j, stop, opt, n1, n2;
  double x1;

  //--- Verify filenames arguments --------------------------------------------
  if (argc < 3) {
    cout << "Usage : test_tdelta <char_filename> <item_filename> [<specs_filename>]" << endl;
    return 0;
  }
  //--- Creates CharList, ItemList and Specs objects and parses the
  //    corresponding text files ----------------------------------------------
  if (argc == 3)
    Example = new tDelta(argv[1], argv[2]);   // no specifications file
  else
    Example = new tDelta(argv[1], argv[2], argv[3]);
  if (!(Example->chars->is_parsed() && Example->items->is_parsed())) {
    cout << "Error parsing characters and/or items description files" << endl;
    delete Example;
    return 1;
  }
  if (Example->specs && !Example->specs->is_parsed()) {
    cout << "Error parsing specifications file" << endl;
    delete Example;
    return 1;
  }
  //--- Displays header -------------------------------------------------------
  cout << "==================" << endl;
  cout << "Free Delta Project" << endl;
  cout << "==================" << endl;
  cout << "Test of tDelta class" << endl << endl;
  cout << "File \"" << Example->chars->get_filename() << "\" parsed" << endl;
  cout << "File \"" << Example->items->get_filename() << "\" parsed" << endl;
  if (Example->specs)
    cout << "File \"" << Example->specs->get_filename() << "\" parsed" << endl;
  cout << endl;
  stop = 0;
  while (!stop) {
    //--- Menu ----------------------------------------------------------------
    cout << "1. Character list" << endl;
    cout << "2. Retrieve a character" << endl;
    cout << "3. Item list" << endl;
    cout << "4. Retrieve an item" << endl;
    cout << "5. Item/character-value matching" << endl;
    cout << "6. Identification" << endl;
    cout << "7. Specifications" << endl;
    cout << "8. Retrieve all (debuging)" << endl;
    cout << "9. Exit" << endl;
    cout << "Your choice : ";
    scanf("%d", &opt);
    switch (opt) {
      case 1 :  //----- Displays all characters features -----------------
        cout << endl << "CHARACTERS LIST :" << endl;
        for (i=1; i<=Example->chars->get_chars_nb(); i++)
          cout << i << " : " << Example->chars->get_char_feature(i) << endl;
        cout << "*** " << Example->chars->get_chars_nb() << " characters ***" << endl << endl;
        break;
      case 2 :  //----- Displays information about one character ---------
        cout << "Enter character number to retrieve : ";
        scanf("%d", &opt);
        if ((opt < 1) || (opt > Example->chars->get_chars_nb()))
          cerr << "Invalid character number" << endl;
        else {
          //--- Character type
          cout << endl << "Type : ";
          switch (Example->chars->get_char_type(opt)) {
            case CT_UM :
              cout << "unordered multistate" << endl;
              break;
            case CT_OM :
              cout << "ordered multistate" << endl;
              break;
            case CT_IN :
              cout << "integer numeric" << endl;
              break;
            case CT_RN :
              cout << "real numeric" << endl;
              break;
            case CT_TE :
              cout << "text/comment" << endl;
          }
          //--- Feature
          cout << "Feature : " << Example->chars->get_char_feature(opt) << endl;
          //--- State list or unit
          if (Example->chars->get_char_type(opt) & CT_UM)
            for (j=1; j<=Example->chars->get_states_nb(opt); j++)
              cout << "State " << j << " : " << Example->chars->get_state(opt,j) << endl;
          else
            if (Example->chars->get_char_type(opt) & CT_IN)
              cout << "Unit : " << Example->chars->get_char_unit(opt) << endl;
        }
        cout << endl;
        break;
      case 3 :  //----- Displays all items names -------------------------
        cout << endl << "ITEMS LIST :" << endl;
        for (i=1; i<=Example->items->get_items_nb(); i++)
          cout << i << " : " << Example->items->get_item_name(i, 0) << endl;
        cout << "*** " << Example->items->get_items_nb() << " items ***" << endl << endl;
        break;
      case 4 :  //----- Displays information about an item ---------------
        cout << "Enter item number to retrieve : ";
        scanf("%d", &opt);
        if ((opt < 1) || (opt > Example->items->get_items_nb()))
          cerr << "Invalid item number" << endl;
        else {
        // Name
          cout << endl << "Item name : " << endl;
          cout << "  " << Example->items->get_item_name(opt) << endl;
        // Attributes
          cout << "Attributes : " << endl;
          for (j=1; j<=Example->items->get_attributes_nb(opt); j++)
            cout << "  " << Example->items->get_attribute(opt,j) << endl;
        }
        cout << endl;
        break;
      case 5 :  //----- Matching item with character value ---------------
        cout << "Enter item number : ";
        scanf("%d", &n1);
        if ((n1 < 1) || (n1 > Example->items->get_items_nb())) {
          cerr << "Invalid item number" << endl;
          break;
        }
        cout << "Enter character number : ";
        scanf("%d", &n2);
        if ((n2 < 1) || (n2 > Example->chars->get_chars_nb())) {
          cerr << "Invalid character number" << endl;
          break;
        }
        cout << "Enter character state/value : ";
        scanf("%lf", &x1);
        // Display item name
        cout << endl << "Item " << Example->items->get_item_name(n1) << " is ";
        // Compare and display result
        if (!Example->items->matches(n1, n2, &x1))
          cout << "not ";
        cout << "matching with" << endl;
        // Display character and state/value
        cout << "character : " << Example->chars->get_char_feature(n2) << endl;
        cout << "state/value : " << x1;
        if (Example->chars->get_char_type(n2) & CT_UM)
          cout << " " << Example->chars->get_state(n2, (int) x1);  // state
        else
          if (Example->chars->get_char_type(n2) & CT_IN)
            cout << " " << Example->chars->get_char_unit(n2);      // unit
        cout << endl << endl;
        break;
      case 6 :  //----- Item identification ------------------------------
        identification(Example);
        break;
      case 7 :  //----- Displays specification file content --------------
        if (Example->specs)
          Example->specs->retrieve_all();
        else
          cout << "No specifications file" << endl;
        cout << endl;
        break;
      case 8 :  //----- Displays all information stored after parsing (debuging)
        Example->items->retrieve_all();
        Example->chars->retrieve_all();
        if (Example->specs)
          Example->specs->retrieve_all();
        break;
      case 9 :  //----- Exit ---------------------------------------------
        stop = 1;
    }  // switch (opt)
  }  // while (!stop)

  delete Example;
  return 0;
}
